//
//  bit_vector.c 
//  
//
//  Created by Matt Stern on 11/23/13.
//
//

#include <inttypes.h>

uint8_t getBit(uint8_t n, size_t i){
    //Returns the `i`th bit of `n`, 7th bit is MSb
    return ( n & (1 << i) ) ? 1 : 0;
}

void setBit(uint8_t* n_ptr, size_t i, uint8_t val){
    //Sets the `i`th bit of *n to `val`, 7th bit is MSb
    if (val){
		*n_ptr |= (1 << i);
    } else {
		*n_ptr &= ~(1 << i);
    }
}

uint8_t get1(uint8_t arr[], size_t i){
    //Get the `i`th bit from `arr`
    uint32_t idx   = i >> 3;  //Which byte in the array the bit is in
    uint8_t offset = 7 - (i - (idx << 3)); //Which bit in the byte, 0th is LSb
    return getBit(arr[idx], offset); 
}

void set1(uint8_t* arr, size_t i, uint8_t val){
	//Sets the `ith` bit in `arr` to `val`
    uint32_t idx   = i >> 3;  //Which byte in the array the bit is in
    uint8_t offset = 7 - (i - (idx << 3)); //Which bit in the byte, 0th is LSb
    setBit(arr + idx, offset, val); 
}

uint8_t get7(uint8_t arr[], size_t i){
    //Gets a contiguous 7 bits starting at bit `i` in `arr`
    uint8_t result = 0;
    for (uint8_t j = 0; j < 7; j++){
		result |= get1(arr, i + j) << (6-j);
    }
    return result;
}

void set7(uint8_t arr[], size_t i, uint8_t val){
    //Sets 7 continguous bits starting at bit `i` to the 7 LSb's
    //of `val`
    for (uint8_t j = 0; j < 7; j++){
		set1(arr, i + j, getBit(val, 6-j));
    }
}



