//
//  bit_vector_test.c
//  
//
//  Created by Matt Stern on 11/24/13.
//
//

#include <stdio.h>

#include "bit_vector.c"
#include "uart.h"
#include "uart.c"

// UART file descriptor
// putchar and getchar are in uart.c
FILE uart_str = FDEV_SETUP_STREAM(uart_putchar, uart_getchar, _FDEV_SETUP_RW);

//--------------------Helper functions-------------------------------
void cmp8(uint8_t exp, uint8_t act){
	fprintf(stdout, "Expected: %i, Actual: %i\n", exp, act);
}

void cmp32(uint32_t exp, uint32_t act){
	fprintf(stdout, "Expected: %ld, Actual: %ld\n", exp, act);
}

//--------------------Begin tests-------------------------------------

uint8_t get1AndSet1_test(void){
    uint8_t passed = 1;
    uint8_t arr[100];
    
    set1(arr, 0, 1);
    passed &= get1(arr, 0) == 1;
    passed &= arr[0] == 0x80;
    
    set1(arr, 0, 0);
    passed &= get1(arr, 0) == 0;
    passed &= arr[0] == 0x0;
    
    set1(arr, 6, 1);
    passed &= get1(arr, 6) == 1;
    passed &= arr[0] == 0x02;
    
    return passed;
}

uint8_t get7AndSet7_test(void) {
    uint8_t passed = 1;
    uint8_t arr[100];
    
    arr[0] = 0x02;
    passed &= get1(arr, 6) == 1;
    passed &= get7(arr, 0) == 0x01;
    
    arr[0] = 0xff;
    passed &= get7(arr, 0) == 0x7f;
    
    arr[0] = 0x07;
    passed &= get7(arr, 0) == 0x03;
    passed &= get7(arr, 7) == 0x40;
    
    set7(arr, 0, 1);
    set7(arr, 7, 0);
    passed &= arr[0] == 0x02;
    
    set7(arr, 0, 0);
    passed &= arr[0] == 0x00;
    set7(arr, 4, 0x7f);
    passed &= arr[0] == 0x0f;
    passed &= arr[1] == 0xe0;

	uint8_t buffer[10];
	uint8_t result[10];

	arr[0] = 0x0f;
	arr[1] = 0xf1;
	arr[2] = 0x07;

	buffer[0] = get7(arr, 0);
	buffer[1] = get7(arr, 7);
	buffer[2] = get7(arr, 14);
	buffer[3] = get7(arr, 21);


	set7(result, 0,  buffer[0]);
	set7(result, 7,  buffer[1]);
	set7(result, 14, buffer[2]);
	set7(result, 21, buffer[3]);

	cmp8(0x0f, result[0]);
	cmp8(0xf1, result[1]);
	cmp8(0x07, result[2]);

	passed &= result[0] == 0x0f;
	passed &= result[1] == 0xf1;
	passed &= result[2] == 0x07;

    //Basic sweep
    uint16_t idx = 0;
    for (uint8_t i = 0; i < 50; i++){
        set7(arr, idx, i);
        idx += 7;
    }
    idx = 0;
    for (uint8_t i = 0; i < 50; i++){
        passed &= get7(arr, idx) == i;
        idx += 7;
    }
    return passed;
}

int main(void){
    uart_init();
    
	stdout = stdin = stderr = &uart_str;
    fprintf(stdout, "Initializing teacher_test.c\n");
    
    uint8_t test_passed;
    
	test_passed = get7AndSet7_test();
    fprintf(stdout, "get7AndSet7_test: %i\n", test_passed);
    
	fprintf(stdout, "\n");
    
}
