//
//  dds_test.c
//  
//
//  Created by Matt Stern on 11/12/13.
//
//

#define F_CPU 16000000
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <inttypes.h>
#include <stdio.h>

#include "dds.c"
#include "uart.h"
#include "uart.c"

// UART file descriptor
// putchar and getchar are in uart.c
FILE uart_str = FDEV_SETUP_STREAM(uart_putchar, uart_getchar, _FDEV_SETUP_RW);

//--------------------Helper functions-------------------------------
void cmp8(uint8_t exp, uint8_t act){
	fprintf(stdout, "Expected: %i, Actual: %i\n", exp, act);
}

void cmp32(uint32_t exp, uint32_t act){
	fprintf(stdout, "Expected: %ld, Actual: %ld\n", exp, act);
}

void clear_arrays(void){
	for (uint16_t i = 0; i < MAX_CH; i++){
		for (uint16_t j = 0; j < MAX_NOTES; j++){
			inc_main[i][j] = 0;
			channel_on[i][j] = 0;
		}
	}
}

void setSampleNotes(void){
	buttonNotes[0x0] = 60; //C4
	buttonNotes[0x1] = 62; //D4
	buttonNotes[0x2] = 64; //E4
	buttonNotes[0x3] = 65; //F4
	buttonNotes[0x4] = 67; //G4
	buttonNotes[0x5] = 69; //A4
	buttonNotes[0x6] = 71; //B4
	buttonNotes[0x7] = 72; //C5
	buttonNotes[0x8] = 74; //D5
	buttonNotes[0x9] = 76; //E5
	buttonNotes[0xa] = 77; //F5
	buttonNotes[0xb] = 79; //G5
	buttonNotes[0xc] = 81; //A5
	buttonNotes[0xd] = 83; //B5
	buttonNotes[0xe] = 84; //C6
	buttonNotes[0xf] = 86; //D6
}

//--------------------Begin tests-------------------------------------

uint8_t find_note_off_test(void){
	uint8_t passed = 1;
	clear_arrays();

	//Initialize test data
	inc_main[0][0] = 100;
	inc_main[0][1] = 0;
	inc_main[0][2] = 200;
	inc_main[1][0] = 200;
	inc_main[1][1] = 300;
	inc_main[1][2] = 400;

	channel_on[0][0] = 1;
	channel_on[0][1] = 0;
	channel_on[0][2] = 1;
	channel_on[1][0] = 1;
	channel_on[1][1] = 1;
	channel_on[1][2] = 0;

	passed &= find_note_off(0, 100) == 0;
	passed &= find_note_off(0, 200) == 2;
	passed &= find_note_off(1, 200) == 0;
	passed &= find_note_off(1, 300) == 1;
	passed &= find_note_off(1, 400) == -1;
	
	channel_on[1][2] = 1;
	passed &= find_note_off(1, 400) == 2;

	return passed;
}

uint8_t find_free_note_test(void){
	uint8_t passed = 1;
	clear_arrays();

	//Initialize test data
	inc_main[0][0] = 100;
	inc_main[0][1] = 0;
	inc_main[0][2] = 200;
	inc_main[1][0] = 200;
	inc_main[1][1] = 300;
	inc_main[1][2] = 400;

	channel_on[0][0] = 0;
	channel_on[0][1] = 0;
	channel_on[0][2] = 1;
	channel_on[1][0] = 1;
	channel_on[1][1] = 1;
	channel_on[1][2] = 0;

	passed &= find_free_note(0, 100) == 0;
	channel_on[0][0] = 1;
	passed &= find_free_note(0, 100) == -1;

	passed &= find_free_note(0, 150) == 1;
	channel_on[0][1] = 1;
	passed &= find_free_note(0, 175) == -1;

	passed &= find_free_note(1, 150) == 2;
	channel_on[1][2] = 1;
	passed &= find_free_note(1, 170) == -1;

	return passed;
}

uint8_t playButtonPress_test(void){
	uint8_t passed = 1;
	clear_arrays();
	setSampleNotes();

	buttons = 0;
	playButtonPress();
	
	passed &= inc_main[MAX_CH-1][0] == 0;
	passed &= inc_main[MAX_CH-1][1] == 0;
	passed &= inc_main[MAX_CH-1][2] == 0;

	passed &= channel_on[MAX_CH-1][0] == 0;
	passed &= channel_on[MAX_CH-1][1] == 0;
	passed &= channel_on[MAX_CH-1][2] == 0;

	buttons = 0x0003;
	playButtonPress();

	passed &= inc_main[MAX_CH-1][0] == midiIncrTable[buttonNotes[0]];
	passed &= inc_main[MAX_CH-1][1] == midiIncrTable[buttonNotes[1]];
	passed &= inc_main[MAX_CH-1][2] == 0;

	passed &= channel_on[MAX_CH-1][0] == 1;
	passed &= channel_on[MAX_CH-1][1] == 1;
	passed &= channel_on[MAX_CH-1][2] == 0;

	buttons = 0;
	playButtonPress();
	
	passed &= inc_main[MAX_CH-1][0] == midiIncrTable[buttonNotes[0]];
	passed &= inc_main[MAX_CH-1][1] == midiIncrTable[buttonNotes[1]];
	passed &= inc_main[MAX_CH-1][2] == 0;

	passed &= channel_on[MAX_CH-1][0] == 0;
	passed &= channel_on[MAX_CH-1][1] == 0;
	passed &= channel_on[MAX_CH-1][2] == 0;

	//Too many buttons pressed, only the first 3 (starting from LSB will be pressed)
	buttons = 0x001e;
	playButtonPress();
	
	passed &= inc_main[MAX_CH-1][0] == midiIncrTable[buttonNotes[1]];
	passed &= inc_main[MAX_CH-1][1] == midiIncrTable[buttonNotes[2]];
	passed &= inc_main[MAX_CH-1][2] == midiIncrTable[buttonNotes[3]];

	passed &= channel_on[MAX_CH-1][0] == 1;
	passed &= channel_on[MAX_CH-1][1] == 1;
	passed &= channel_on[MAX_CH-1][2] == 1;

	return passed;
}


int main(void){
    uart_init();
	initDDS();
	initTables();

	stdout = stdin = stderr = &uart_str;
    fprintf(stdout, "Initializing midi_lib_test.c\n");
    
    uint8_t test_passed;

	test_passed = find_note_off_test();
    fprintf(stdout, "find_note_off_test: %i\n", test_passed);

	test_passed = find_free_note_test();
    fprintf(stdout, "find_free_note_test: %i\n", test_passed);

	test_passed = playButtonPress_test();
    fprintf(stdout, "playButtonPress_test: %i\n", test_passed);

	fprintf(stdout, "\n");

}
