//
//  df_stream_test.c
//  
//
//  Created by Matt Stern on 11/15/13.
//
//

#define F_CPU 16000000
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <inttypes.h>
#include <stdio.h>

#include "df_stream.c"
#include "uart.h"
#include "uart.c"

// UART file descriptor
// putchar and getchar are in uart.c
FILE uart_str = FDEV_SETUP_STREAM(uart_putchar, uart_getchar, _FDEV_SETUP_RW);

//--------------------Helper functions-------------------------------
void cmp8(uint8_t exp, uint8_t act){
	fprintf(stdout, "Expected: %i, Actual: %i\n", exp, act);
}

void cmp32(uint32_t exp, uint32_t act){
	fprintf(stdout, "Expected: %ld, Actual: %ld\n", exp, act);
}

//--------------------Begin tests-------------------------------------

volatile df_stream stream;
volatile df_stream* stream_ptr = &stream;

uint8_t df_get_and_set_test(void){
    uint8_t passed = 1;
    
    for (uint16_t i = 0; i < 1000; i++){
        df_set(stream_ptr, i, (uint8_t) i);
    }
    
    for (uint16_t i = 0; i < 1000; i++){
        passed &= df_get(stream_ptr, i) == i;
    }
    
    for (uint16_t i = 0; i < 1000; i++){
        df_set(stream_ptr, i, (uint8_t) (i + 4));
    }
    
    for (uint16_t i = 0; i < 1000; i++){
        passed &= df_get(stream_ptr, i) == (i + 4);
    }
    
    return passed;
}

uint8_t df_cpy_test(void){
    uint8_t passed = 1;
    
    volatile df_stream new_stream;
    volatile new_stream_ptr = &new_stream;
    
    for (uint16_t i = 0; i < 1000; i++){
        df_set(stream, i, i);
    }
    
    df_cpy(new_stream_ptr, stream_ptr, 0, 500, 300);
    
    for (uint16_t i = 0; i < 300; i++){
        passed &= df_get(new_stream_ptr, i) == (i + 500);
    }
    
    return passed;
}


int main(void){
    uart_init();
    
	stdout = stdin = stderr = &uart_str;
    fprintf(stdout, "Initializing midi_lib_test.c\n");
    
    uint8_t test_passed;
 
    test_passed = df_get_and_set_test();
    fprintf(stdout, "df_get_and_set_test: %i\n", test_passed);
    
    test_passed = df_cpy_test();
    fprintf(stdout, "df_cpy_test: %i\n", test_passed);

	fprintf(stdout, "\n");
}

