#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <inttypes.h>
#include <util/delay.h>
#include <stdio.h>
#include <string.h>

#include "midi_lib.c"
#include "ir_comm.c"

#include "uart.c"
#include "uart.h"
FILE uart_str = FDEV_SETUP_STREAM(uart_putchar, uart_getchar, _FDEV_SETUP_RW);

// UART file descriptor
// putchar and getchar are in uart.c
volatile uint8_t clock_ticks;
#define CLOCK_PD 8
uint8_t buffer[256];
uint8_t ir_comm_mode = 1;

volatile uint8_t test_done = 0;

/** HELPER METHODS **/
void set_ir_tx_buffer(char input[]) {
	sprintf(ir_tx_buffer, input);
}

void set_ir_rx_buffer(char input[]) {
	sprintf(ir_rx_buffer, input);
}

void init_everything(void) {
   	// make B.3 an output
   	DDRB = (1<<PINB3) ;
   
   	//init the UART -- uart_init() is in uart.c
   	uart_init();
   	stdout = stdin = stderr = &uart_str;
   	printf("Engine initialized...\n");

	// timer 1 ticks at 8000 Hz or 125 microsecs period=2000 ticks
	OCR1A = 1999 ; // 2000 ticks
	TIMSK1 = (1<<OCIE1A) ;
	TCCR1B = 0x09; 	//full speed; clear-on-match
 	TCCR1A = 0x00;	//turn off pwm and oc lines
  
	// Timer 2 stuff for IR Transmission
	// IR carrier generator
	// set up timer 2 for square wave with NO ISR
	// 56000 Hz => 1/2 cycle 8.928 microsec = 143 cycles
	// Connect D7 thru resistor to (+)IR_LED(-) to TXD:: D7--/\/\--|>|----TXD
	//
	OCR2A = 142 ; //143 cycles/half-period
	//  count at full rate
	TCCR2B =  1;	
	// set to toggle OC2A, clear on match,
	TCCR2A = (1<<COM2A0) | (1<<WGM21) ;
	// PORT D.7 is OC2A
	DDRD = (1<<PIND7) ;	
	//********************
  
	// turn on all ISRs
    sei() ;

}

void genSampleMidi(uint16_t space){
	//Generate a sample MIDI stream
    midi_stream_idx = 0;
	
	uint8_t toggle = 1;
    addQuarterFrameMsg();

	uint8_t c_scale[16] = {60, 62, 64, 65, 67, 69, 71, 72, 72, 71, 69, 67, 65, 64, 62, 60}; 

   	for (uint16_t i = 0; i < 32; i++){
     	midiEvent(toggle ? 0x9 : 0x8, 0x0, c_scale[i>>1], 0xbb);
		midiEvent(toggle ? 0x9 : 0x8, 0x0, c_scale[i>>1] + 12, 0xbb); //+1 octave
		//midiEvent(toggle ? 0x9 : 0x8, 0x0, c_scale[i>>1] - 12, 0xbb); //-2 the octave

        toggle ^= 1;       
       	for (uint16_t j = 0; j < space; j++){
        	incrementSMPTE();
         	if (isQuarterFrame()) addQuarterFrameMsg();
       	}
   	}
    
    finalizeMidiStream();
	//saveRecording();
}

uint8_t test_get_num_packets_necessary(void) {
	uint8_t result = 0x01;
	genSampleMidi(50); // Length = 296
	
	printf("Start test_get_num_packets_necessary\n");
	
	
	genSampleMidi(22);
	result &= get_num_packets_necessary(midi_stream) == 5;
	printf("length: %i\n", getTrackLen(midi_stream));
	printf("num: %i\n", get_num_packets_necessary(midi_stream));
	
	genSampleMidi(21);
	result &= get_num_packets_necessary(midi_stream) == 5;
	printf("length: %i\n", getTrackLen(midi_stream));
	printf("num: %i\n", get_num_packets_necessary(midi_stream));

	genSampleMidi(250); // Length = 1304
	result &= get_num_packets_necessary(midi_stream) == 11;
	printf("length: %i\n", getTrackLen(midi_stream));
	printf("num: %i\n", get_num_packets_necessary(midi_stream));

	genSampleMidi(222); // Length = 1876
	result &= get_num_packets_necessary(midi_stream) == 11;
	printf("length: %i\n", getTrackLen(midi_stream));
	printf("num: %i\n", get_num_packets_necessary(midi_stream));

	genSampleMidi(300); // Length = 2572
	result &= get_num_packets_necessary(midi_stream) == 13;
	printf("length: %i\n", getTrackLen(midi_stream));
	printf("num: %i\n", get_num_packets_necessary(midi_stream));

	genSampleMidi(450); // Length = 3508
	result &= get_num_packets_necessary(midi_stream) == 17;
	printf("length: %i\n", getTrackLen(midi_stream));
	printf("num: %i\n", get_num_packets_necessary(midi_stream));

	genSampleMidi(501); // Length = 4526
	result &= get_num_packets_necessary(midi_stream) == 19;
	printf("length: %i\n", getTrackLen(midi_stream));
	printf("num: %i\n", get_num_packets_necessary(midi_stream));

	return result;
}

uint8_t test_get_data_from_midi(uint16_t length_of_midi) {
	uint8_t result = 0x01;	
	genSampleMidi(length_of_midi); // Length = 3508
	
	uint8_t data[getTrackLen(midi_stream)];
	uint16_t data_index = 0;
	packet_id = 1;
	
	for (uint8_t i = 0; i < get_num_packets_necessary(midi_stream); i++) {
		uint8_t buffer[payload_size + 1];
		write_to_tx(buffer);
		read_from_rx(buffer);
		packet_id++;
	}

	printf("len: %ld\n", getTrackLen(midi_stream));
	for (int i = 0; i < getTrackLen(midi_stream); i++){
		if (pb_stream[i] != midi_stream[i]){
			for (int j = -5; j <= 5; j++){
				printf("midi[%i] = %i, pb[%i] = %i\n", i + j, midi_stream[i+j], i + j, pb_stream[i+j]);
			}
		}
	}

	int n = memcmp(pb_stream, midi_stream, getTrackLen(midi_stream));
	if (n>0) {
		return 0;
	}
	else if (n < 0) {
		return 0;
	}
	else if (n == 0) {
		return 1;
	}
}

/*uint8_t test_mem_to_string(uint16_t length_of_midi) {
	uint8_t result = 0x01;
	
	printf("Start test_mem_to_string\n");
	
	genSampleMidi(length_of_midi);
	
	uint16_t shifted_i = 0;
	char data[payload_size + 1];
	uint16_t data_index = 0;
	
	uint8_t* chunk = get_data_from_midi(midi_stream, 0, payload_size);
	
	memcpy(&data[0], chunk, payload_size);
	data[64] = '\0';
	
	// for (uint8_t i = 0; i < get_num_packets_necessary(midi_stream); i++) {
		// shifted_i = (uint16_t) i << 6;
		// uint8_t* chunk = get_data_from_midi(midi_stream, shifted_i, payload_size);
		
		// memcpy(&data[data_index], chunk, payload_size);	
		// data_index = data_index + 64;
	// }
	
	//sprintf(data, "%s", chunk);
	//char data[payload_size + 1];
	
	//data = (char *) chunk;
	for (uint8_t i = 0; i < 64; i++) {
		printf("data: %i: %i\n", i, data[i]);
		printf("chunk: %i: %i\n", i, chunk[i]);
	}
	int n = memcmp(data, chunk, payload_size);
	if (n>0) {
		return 0;
	}
	else if (n < 0) {
		return 0;
	}
	else if (n == 0) {
		return 1;
	}

} */

uint8_t test_syn(uint16_t length_of_midi) {
	uint8_t result = 0;
	genSampleMidi(length_of_midi);
	char ir_rx_data_test[buffer_size];
	send_syn();
	
	while(1) {
		uint8_t status = receive_syn(ir_rx_data_test);
		//printf("status: %i\n", status);
		//printf("get_num: %i\n", get_num_packets_necessary(midi_stream));
		//for (int i = 0; i < buffer_size; i++){
		//	printf("ir_rx_test[%i] = %i\n", i, ir_rx_data_test[i]);
		//}
		if (status == 6) {
			if (ir_rx_data_test[0] == get_num_packets_necessary(midi_stream)) {
				result = 1;
			}
			else {
				result = 0;
			}
			break;
		} else {
			
		}
	}
	test_done = 1;
	return result;
}

uint8_t test_syn_ack(uint16_t length_of_midi) {
	uint8_t result = 0;
	genSampleMidi(length_of_midi);
	char ir_rx_data_test[69];
	send_syn_ack();
	
	while(1) {
		uint8_t status = receive_syn_ack(ir_rx_data_test);
		if (status == 6) {
			result = 1;
			break;
		}
		else {
			//printf("status: %i\n", status);
		}
	}
	test_done = 1;
	return result;
}

uint8_t test_fin(uint16_t length_of_midi) {
	uint8_t result = 0;
	genSampleMidi(length_of_midi);
	char ir_rx_data_test[69];
	send_fin();
	
	while(1) {
		uint8_t status = receive_fin(ir_rx_data_test);
		if (status == 6) {
			result = 1;
			break;
		}
		else {
			//printf("status: %i\n", status);
		}
	}
	test_done = 1;
	return result;
}


uint8_t test_fin_ack(uint16_t length_of_midi) {
	uint8_t result = 0;
	genSampleMidi(length_of_midi);
	char ir_rx_data_test[69];
	send_fin_ack();
	
	while(1) {
		uint8_t status = receive_fin_ack(ir_rx_data_test);
		if (status == 6) {
			result = 1;
			break;
		}
		else {
			//printf("status: %i\n", status);
		}
	}
	test_done = 1;
	return result;
}

uint8_t test_packet(uint16_t length_of_midi) {
	uint8_t result = 0;
	genSampleMidi(length_of_midi);
	
	uint8_t last_pack_id = get_num_packets_necessary(midi_stream);
	
	for (uint8_t i = 1; i <= last_pack_id; i++) {
		char ir_rx_data_test[69];
		send_packet();
		
		while(1) {
			uint8_t status = receive_packet(ir_rx_data_test);
			if (status == 6) {
				result = 1;
				//printf("loop: %i\n", i);
				break;
			}
			else {
				//printf("status: %i", status);
			}
		}
		packet_id++;
		_delay_ms(1000);
	}
	test_done = 1;

	for (int i = 0; i < payload_size; i++) {
		
	}
	
	//printf("ir_rx_data: %i, answer len: %i\n", getTrackLen(pb_stream), getTrackLen(midi_stream));
	//printf("ir_rx_data: %s\n", pb_stream);
	//printf("answer: %s\n", midi_stream);
	
	
	int n = memcmp(pb_stream, midi_stream, payload_size);
	if (n>0) {
		return 0;
	}
	else if (n < 0) {
		return 0;
	}
	else if (n == 0) {
		return 1;
	}
	
	return result;
}

#define SEND_SYN 0
#define RECEIVE_SYN_ACK 1
#define SEND_PACKET 2
#define RECEIVE_PACKET_ACK 3
#define SEND_FIN 4
#define RECEIVE_FIN_ACK 5


ISR (TIMER1_COMPA_vect){ //Fs = 8000
    //Generate 1 msec avg time base for MAIN	
	clock_ticks--;
	if (clock_ticks == 0){
		//Timebase logic
		clock_ticks = CLOCK_PD;
		ir_time++;

		if (ir_comm_mode) {
			//**********************
			// send an ir char if tx is ready and still char in buffer to send
			// and USART is ready
			if ( ir_tx_ready && (duplex || is_tx_state()) ){ //&& ir_tx_buffer[ir_tx_count]>0
				if (UCSR1A & (1<<UDRE1)) UDR1 = ir_tx_buffer[ir_tx_count++];
				if (ir_tx_count >= buffer_size) ir_tx_ready = 0; // buffer overrun
			}
			
			//**********************
			// recv an ir char if data ready 
			// append character to the received string
			if ( (UCSR1A & (1<<RXC1)) && (duplex || !is_tx_state()) ) {
				uint8_t c = UDR1;
				if (ir_rx_count != 0 || c == start_token){
					ir_rx_buffer[ir_rx_count++] = c;

					if (ir_rx_count >= buffer_size) {
						ir_rx_ready = 1;
						ir_rx_buffer[buffer_size-1] = 0x00;
						ir_rx_count = buffer_size -1 ; //???
					}
				}
			}
		}
	}
}
