//
//  teacher.c
//  
//
//  Created by Matt Stern on 11/19/13.
//
//

#include <stdio.h>

#define SENSITIVITY_MSEC 150
#define MAX(A,B) ((A > B) ? A : B)

//Variables to keep track of current and next QFM indices and
//the number of the QFMs themselves
int32_t mstream_idx ;
int32_t stream_idx;
int32_t num_QFM_mstream;
int32_t num_QFM_stream;
int32_t next_QFM_stream;
int32_t next_QFM_mstream;

void initGlobals(void){
    //Initialize global variables
    mstream_idx = MIDI_STREAM_OVHD;
    stream_idx = MIDI_STREAM_OVHD;
    num_QFM_mstream = 0;
    num_QFM_stream = 0;
    next_QFM_stream = 0;
    next_QFM_mstream = 0;
}

void findStartSensWindow(int32_t mstream_len){
    //Finds the idx of the start of the sensitivity window in `midi_stream`, defined as the maximum
    // of SENSITIVITY_MSEC seconds before the current QFM in the reference stream
    //and time 0
	while (num_QFM_mstream < MAX(0, num_QFM_stream - SENSITIVITY_MSEC)){
		next_QFM_mstream = indexOfQFM(midi_stream, mstream_idx, mstream_len - mstream_idx);
		if (next_QFM_mstream == -1){ //We've reached the end of `midi_stream`
			return;
		} else if (mstream_idx == next_QFM_mstream){ //We were already at a QFM
		 	mstream_idx = next_QFM_mstream + 2;
			num_QFM_mstream++;
		} else {
			mstream_idx = next_QFM_mstream; //We were in between QFMs
		}
	}
}

uint8_t findMatchingEvent(int32_t mstream_len, uint8_t event, uint8_t param_1){
    //Searches within SENSTIVITY_MSEC msec on either side of `num_QFM_stream` for
    //`event` with first param `param_1` in `midi_stream`
    
    findStartSensWindow(mstream_len);

	//Find the first event within the sensitivity window and compare it to the one passed in by `stream`
    while (num_QFM_mstream < num_QFM_stream + SENSITIVITY_MSEC){
		next_QFM_mstream = indexOfQFM(midi_stream, mstream_idx, mstream_len - mstream_idx);
		if (next_QFM_mstream == -1){
			return 0;
		}
	
        if (mstream_idx < next_QFM_mstream){
            //Extrac tthe event and parameter at the first encountered event
            uint8_t m_event   = (midi_stream[mstream_idx] & 0xf0) >> 4;
			uint8_t m_param_1 = midi_stream[mstream_idx+1];
			
            //Start the next call of this function at the next event/QFM
            mstream_idx += 3;
            if (mstream_idx != next_QFM_mstream){
                num_QFM_mstream--;
            }
            return (event == m_event) && (param_1 == m_param_1);
        } else {
        	num_QFM_mstream++;
			mstream_idx += 2;
		}
    }
	return 0;
}

void calcScore(uint16_t* corr_ptr, uint16_t* incorr_ptr, uint8_t* stream, int32_t stream_len, int32_t mstream_len){
    //Compares `midi_stream` to `stream` using a comparison algorithm explained in
    //http://people.ece.cornell.edu/land/courses/ece4760/FinalProjects/f2013/mjs476_ajl264/mjs476_ajl264/index.html
    
    uint16_t corr = 0;
    uint16_t incorr = 0;
    initGlobals();
    
    while ((next_QFM_stream = indexOfQFM(stream, stream_idx, stream_len - stream_idx)) != -1){
		while (stream_idx < next_QFM_stream){
            uint8_t event   = (stream[stream_idx] & 0xf0) >> 4;
			uint8_t param_1 = stream[stream_idx+1];
            if (event == 0xc0){ //Instrument changes aren't considered
                continue;
            }
            else if (findMatchingEvent(mstream_len, event, param_1)){
                corr++;
            } else {
                incorr++;
            }
            stream_idx += 3;
        }
		stream_idx += 2;
        num_QFM_stream++;
    }
    
    *corr_ptr = corr;
    *incorr_ptr = incorr;
}

