//
//  teacher_test.c
//  
//
//  Created by Matt Stern on 11/20/13.
//
//

#include <stdio.h>

#define F_CPU 16000000
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <inttypes.h>
#include <stdio.h>
#include <string.h>

#include "midi_lib.c"
#include "teacher.c"
#include "uart.h"
#include "uart.c"

// UART file descriptor
// putchar and getchar are in uart.c
FILE uart_str = FDEV_SETUP_STREAM(uart_putchar, uart_getchar, _FDEV_SETUP_RW);

#define SENSITIVITY_MSEC 250 //Unit tests are dependent on this redefinition

//--------------------Helper functions-------------------------------
void cmp8(uint8_t exp, uint8_t act){
	fprintf(stdout, "Expected: %i, Actual: %i\n", exp, act);
}

void cmp32(uint32_t exp, uint32_t act){
	fprintf(stdout, "Expected: %ld, Actual: %ld\n", exp, act);
}

int16_t genStreams(uint8_t diff_notes, uint8_t skip_note){
	//Generate a sample stream for unit testing
	//This stream has 20 events spaced out at intervals of 50 QFM
	midi_stream_idx = 0;
    uint8_t count = 50;
    uint8_t toggle = 1;
    uint8_t events = 0;
    
    //Every 50 QFM (every 100 bytes), add a new 3 byte event. Thus...
    //QFM 0   => mstream_idx 0
    //QFM 50  => mstream_idx 103
    //QFM 100 => mstream_idx 206
    //etc...
	uint8_t note = 60;
    while (events < 20){
        addQuarterFrameMsg();
        count--;
        if (count == 0){
			if ((!skip_note || (events != 10 && events != 11))){ //Skip the 6th notex
            	toggle ? midiEvent(9, 0, note, 0) : midiEvent(8, 0, note, 0);
			}
            count = 50;
            toggle ^= 1;
            events++;
			if (diff_notes){
				note++;
			}
        }
    }
	addQuarterFrameMsg();
    finalizeMidiStream();
    return MIDI_STREAM_OVHD + getTrackLen(midi_stream);
}

int16_t genStreams2(uint8_t diff_notes, uint8_t skip_note){
	//Generate a sample stream for unit testing
	//This stream has 20 events spaced out at intervals of 800 QFM
	midi_stream_idx = 0;
    uint16_t count = 300;
    uint8_t toggle = 1;
    uint8_t events = 0;
    
    //Every 50 QFM (every 100 bytes), add a new 3 byte event. Thus...
    //QFM 0   => mstream_idx 0
    //QFM 50  => mstream_idx 103
    //QFM 100 => mstream_idx 206
    //etc...
	uint8_t note = 60;
    while (events < 6){
        addQuarterFrameMsg();
        count--;
        if (count == 0){
			if ((!skip_note || (events != 2 && events != 3))){ //Skip the 6th note
            	toggle ? midiEvent(9, 0, note, 0) : midiEvent(8, 0, note, 0);
			}
            count = 300;
            toggle ^= 1;
            events++;
			if (diff_notes){
				note++;
			}
        }
    }
	addQuarterFrameMsg();
    finalizeMidiStream();
    return MIDI_STREAM_OVHD + getTrackLen(midi_stream);
}

//--------------------Begin tests-------------------------------------

uint8_t findStartSensWindow_test(void){
	uint8_t passed = 1;
    uint16_t mstream_len = genStreams(0, 0);
    
	initGlobals();
    findStartSensWindow(mstream_len);
    passed &= num_QFM_mstream == 0;
    passed &= mstream_idx == MIDI_STREAM_OVHD;

	initGlobals();
    num_QFM_stream = 100;
    findStartSensWindow(mstream_len);
    passed &= num_QFM_mstream == 0;
    passed &= mstream_idx == MIDI_STREAM_OVHD;

	initGlobals();
    num_QFM_stream = 250;
    findStartSensWindow(mstream_len);
    passed &= num_QFM_mstream == 0;
    passed &= mstream_idx == MIDI_STREAM_OVHD;

	initGlobals();
    num_QFM_stream = 300;
    findStartSensWindow(mstream_len);
    passed &= num_QFM_mstream == 50;
    passed &= mstream_idx == MIDI_STREAM_OVHD + 100; //100 QFM bytes

    initGlobals(); //Reset mstream_idx
    num_QFM_stream = 300;
    findStartSensWindow(mstream_len);
    passed &= num_QFM_mstream == 50;
    passed &= mstream_idx == MIDI_STREAM_OVHD + 100; //100 QFM bytes

    num_QFM_stream = 301;
    findStartSensWindow(mstream_len);
    passed &= num_QFM_mstream == 51;
    passed &= mstream_idx == MIDI_STREAM_OVHD + 105; //101 QFM bytes + 1 event

    num_QFM_stream = 310;
    findStartSensWindow(mstream_len);
    passed &= num_QFM_mstream == 60;
    passed &= mstream_idx == MIDI_STREAM_OVHD + 123; //120 QFM bytes + 1 event
    
    num_QFM_stream = 500;
    findStartSensWindow(mstream_len);
    passed &= num_QFM_mstream == 250;
    passed &= mstream_idx == MIDI_STREAM_OVHD + 512; //250 QFMs + 4 events

	initGlobals();
	num_QFM_stream = 500;
    findStartSensWindow(mstream_len);
    passed &= num_QFM_mstream == 250;
    passed &= mstream_idx == MIDI_STREAM_OVHD + 512; //250 QFMs + 4 events

    num_QFM_stream = 10000;
    findStartSensWindow(mstream_len);
    passed &= num_QFM_mstream == 1001; //We added an extra QFM at the end
    passed &= mstream_idx == mstream_len;

	return passed;
}

uint8_t findMatchingEvent_test(void){
	uint8_t passed = 1;
	initGlobals();
    uint16_t mstream_len = genStreams(0, 0);

    initGlobals();
    passed &= findMatchingEvent(mstream_len, 9, 60) == 1;
	passed &= mstream_idx == MIDI_STREAM_OVHD + 103;
	passed &= num_QFM_mstream == 50;

    initGlobals();
    passed &= findMatchingEvent(mstream_len, 8, 60) == 0;
	passed &= mstream_idx == MIDI_STREAM_OVHD + 103;
	passed &= num_QFM_mstream == 50;

    initGlobals();
    passed &= findMatchingEvent(mstream_len, 9, 61) == 0;
	passed &= mstream_idx == MIDI_STREAM_OVHD + 103;
	passed &= num_QFM_mstream == 50;

    //Even though we're at the note off event in `stream`, it will
    //still find the note on first. The key to this algorithm is
    //that the events must be matched in increasing order

    initGlobals();
	num_QFM_stream = 100;
    passed &= findMatchingEvent(mstream_len, 9, 60) == 1;
	passed &= mstream_idx == MIDI_STREAM_OVHD + 103;
	passed &= num_QFM_mstream == 50;

    initGlobals();
    passed &= findMatchingEvent(mstream_len, 8, 60) == 0;
	passed &= mstream_idx == MIDI_STREAM_OVHD + 103;
	passed &= num_QFM_mstream == 50;

    initGlobals();
    passed &= findMatchingEvent(mstream_len, 9, 61) == 0;
	passed &= mstream_idx == MIDI_STREAM_OVHD + 103;
	passed &= num_QFM_mstream == 50;

    //Now we can find the note off since we've advanced
    //`num_QFM_mstream` past the note on event
    passed &= findMatchingEvent(mstream_len, 8, 60) == 1;
	passed &= mstream_idx == MIDI_STREAM_OVHD + 206;
	passed &= num_QFM_mstream == 100;

	initGlobals();
	num_QFM_stream = 470; //start at num_QFM_mstream == 220
	passed &= findMatchingEvent(mstream_len, 9, 60) == 1;
	passed &= mstream_idx == MIDI_STREAM_OVHD + 515;
	passed &= num_QFM_mstream == 250;

	return passed;
}

uint8_t calcScore_Test(void){
	uint16_t corr, incorr;
	uint8_t passed = 1;
	uint16_t mstream_len = genStreams(0, 0);
	uint8_t stream[mstream_len<<2]; //4x the length for extra room for later tests
	memcpy((void *) stream, (void *) midi_stream, mstream_len);

	//Identical streams test
	calcScore(&corr, &incorr, stream, mstream_len, mstream_len);
	passed &= corr == 20; //Not 20 because the last event occurs after the last QFM
	passed &= incorr == 0;

	//One wrong event test
	memcpy((void *) midi_stream, (void *) stream, mstream_len);
	midi_stream[MIDI_STREAM_OVHD + 101] = 61; 
	calcScore(&corr, &incorr, stream, mstream_len, mstream_len);
	passed &= corr == 19;
	passed &= incorr == 1;

	//Three wrong events test
	memcpy((void *) midi_stream, (void *) stream, mstream_len);
	midi_stream[MIDI_STREAM_OVHD + 101] = 22; 
	midi_stream[MIDI_STREAM_OVHD + 204] = 51; 
	midi_stream[MIDI_STREAM_OVHD + 307] = 100; 
	calcScore(&corr, &incorr, stream, mstream_len, mstream_len);
	passed &= corr == 17;
	passed &= incorr == 3;

	//-200 ms offset test -- within sensitivity range, same notes
	memcpy((void *) &stream[400 + MIDI_STREAM_OVHD], (void *) &midi_stream[MIDI_STREAM_OVHD], mstream_len); //stream is 400 bytes (200 QFM) late
	for (uint16_t i = MIDI_STREAM_OVHD; i < MIDI_STREAM_OVHD + 400; i = i + 2){
		stream[i] = 0xf1; //Add QFMs to fill the 200 ms of empty space
	}
	calcScore(&corr, &incorr, stream, mstream_len + 400, mstream_len);
	passed &= corr == 20;
	passed &= incorr == 0;

	//Reset everything
	genStreams(0, 0);
	memcpy((void *) stream, (void *) midi_stream, mstream_len);


	//+200 ms offset test -- within sensitivity range, same notes
	memcpy((void *) &midi_stream[400 + MIDI_STREAM_OVHD], (void *) &stream[MIDI_STREAM_OVHD], mstream_len); //midi_stream is 400 bytes (200 QFM) late
	for (uint16_t i = MIDI_STREAM_OVHD; i < MIDI_STREAM_OVHD + 400; i = i + 2){
		midi_stream[i] = 0xf1; //Add QFMs to fill the 200 ms of empty space
	}
	calcScore(&corr, &incorr, stream, mstream_len, mstream_len + 400);
	passed &= corr == 20;
	passed &= incorr == 0;

	//Reset everything
	genStreams(0, 0);
	memcpy((void *) stream, (void *) midi_stream, mstream_len);

	//-300 ms offset test -- outside sensitivity range, same notes
	//If we consider the two streams to be "waves", then the offset of
	//-300 ms causes them to be "out of phase", so all of them are wrong
	memcpy((void *) &stream[600 + MIDI_STREAM_OVHD], (void *) &midi_stream[MIDI_STREAM_OVHD], mstream_len); //stream is 600 bytes (300 QFM) late
	for (uint16_t i = MIDI_STREAM_OVHD; i < MIDI_STREAM_OVHD + 600; i = i + 2){
		stream[i] = 0xf1; //Add QFMs to fill the 200 ms of empty space
	}
	calcScore(&corr, &incorr, stream, mstream_len + 600, mstream_len);
	passed &= corr == 0;
	passed &= incorr == 20;

	//Reset everything
	genStreams(0, 0);
	memcpy((void *) stream, (void *) midi_stream, mstream_len);

	//+300 ms offset test -- within sensitivity range
	//If we consider the two streams to be "waves", then the offset of
	//+300 ms causes them to be "in phase". We have two misses because
	//the `midi_stream` comes in after the sensitive range of the first two
	//events in `stream`
	memcpy((void *) &midi_stream[600 + MIDI_STREAM_OVHD], (void *) &stream[MIDI_STREAM_OVHD], mstream_len); //midi_stream is 600 bytes (300 QFM) late	
	for (uint16_t i = MIDI_STREAM_OVHD; i < MIDI_STREAM_OVHD + 600; i = i + 2){
		midi_stream[i] = 0xf1; //Add QFMs to fill the 200 ms of empty space
	}

	calcScore(&corr, &incorr, stream, mstream_len, mstream_len + 600);
	passed &= corr == 18;
	passed &= incorr == 2;

	//Reset everything, different notes
	genStreams(1, 0);
	memcpy((void *) stream, (void *) midi_stream, mstream_len);

	//-200 ms offset test -- within sensitivity range, different notes
	memcpy((void *) &stream[400 + MIDI_STREAM_OVHD], (void *) &midi_stream[MIDI_STREAM_OVHD], mstream_len); //stream is 400 bytes (200 QFM) late
	for (uint16_t i = MIDI_STREAM_OVHD; i < MIDI_STREAM_OVHD + 400; i = i + 2){
		stream[i] = 0xf1; //Add QFMs to fill the 200 ms of empty space
	}
	calcScore(&corr, &incorr, stream, mstream_len + 400, mstream_len);
	passed &= corr == 20;
	passed &= incorr == 0;

	//Reset everything, different notes
	genStreams(1, 0);
	memcpy((void *) stream, (void *) midi_stream, mstream_len);

	//+200 ms offset test -- within sensitivity range, different notes
	memcpy((void *) &midi_stream[400 + MIDI_STREAM_OVHD], (void *) &stream[MIDI_STREAM_OVHD], mstream_len); //midi_stream is 400 bytes (200 QFM) late
	for (uint16_t i = MIDI_STREAM_OVHD; i < MIDI_STREAM_OVHD + 400; i = i + 2){
		midi_stream[i] = 0xf1; //Add QFMs to fill the 200 ms of empty space
	}
	calcScore(&corr, &incorr, stream, mstream_len, mstream_len + 400);
	passed &= corr == 20;
	passed &= incorr == 0;

	//Reset everything, different notes
	genStreams(1, 0);
	memcpy((void *) stream, (void *) midi_stream, mstream_len);

	//-300 ms offset test -- outside sensitivity range, different notes
	memcpy((void *) &stream[600 + MIDI_STREAM_OVHD], (void *) &midi_stream[MIDI_STREAM_OVHD], mstream_len); //stream is 600 bytes (300 QFM) late
	for (uint16_t i = MIDI_STREAM_OVHD; i < MIDI_STREAM_OVHD + 600; i = i + 2){
		stream[i] = 0xf1; //Add QFMs to fill the 200 ms of empty space
	}
	calcScore(&corr, &incorr, stream, mstream_len + 600, mstream_len);
	passed &= corr == 0;
	passed &= incorr == 20;

	//Reset everything, different notes
	genStreams(1, 0);
	memcpy((void *) stream, (void *) midi_stream, mstream_len);

	//+300 ms offset test -- within sensitivity range
	//If we consider the two streams to be "waves", then the offset of
	//+300 ms causes them to be "in phase". We have two misses because
	//the `midi_stream` comes in after the sensitive range of the first two
	//events in `stream`
	memcpy((void *) &midi_stream[600 + MIDI_STREAM_OVHD], (void *) &stream[MIDI_STREAM_OVHD], mstream_len); //midi_stream is 600 bytes (300 QFM) late	
	for (uint16_t i = MIDI_STREAM_OVHD; i < MIDI_STREAM_OVHD + 600; i = i + 2){
		midi_stream[i] = 0xf1; //Add QFMs to fill the 200 ms of empty space
	}

	calcScore(&corr, &incorr, stream, mstream_len, mstream_len + 600);
	passed &= corr == 0;
	passed &= incorr == 20;

	//Reset everything, different notes, skipped note
	genStreams(1, 0);
	memcpy((void *) stream, (void *) midi_stream, mstream_len);
	genStreams(1, 1);

	//Missed note, notes are too close together, so they get misaligned and
	//all are misses after the misfire on the 10th event (of 19 total events)
	calcScore(&corr, &incorr, stream, mstream_len, mstream_len + 600);
	passed &= corr == 10;
	passed &= incorr == 10;	

	//Reset everything, different notes, skipped note, sufficient interval
	mstream_len = genStreams2(1, 0);
	memcpy((void *) stream, (void *) midi_stream, mstream_len);
	genStreams2(1, 1);

	//Missed note, notes are far enough apart that there is only one
	//missed note (2 event) penalty for the isolated miss
	calcScore(&corr, &incorr, stream, mstream_len, mstream_len + 600);
	passed &= corr == 4;
	passed &= incorr == 2;

	return passed;
}
	

int main(void){
    uart_init();
 
	stdout = stdin = stderr = &uart_str;
    fprintf(stdout, "Initializing teacher_test.c\n");
    
    uint8_t test_passed;
    
	test_passed = findStartSensWindow_test();
    fprintf(stdout, "findStartSensWindow_test: %i\n", test_passed);
    
    test_passed = findMatchingEvent_test();
    fprintf(stdout, "findMatchingEvent_test: %i\n", test_passed);

	test_passed = calcScore_Test();
    fprintf(stdout, "calcScore_test: %i\n", test_passed);
    
	fprintf(stdout, "\n");
    
}
